// Copyright (c) 97 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// MySQL driver for EasySQL API
//
// MySQL is a mostly free but very powerfull SQL database engine.
// See http://www.tcx.se/
//
#include <string.h>
#include <mysql/mysql.h>
#include "DrvHeader.h"

///////////////////////////////////////////////////////////////////////////////
// Data structure for driver
//
struct DS(data) : public easysql_error_reporting
{ MYSQL sql;
  bool connected;
//
  DS(data)();
  ~DS(data)();
};

///////////////////////////////////////////////////////////////////////////////
// Initializing data
//
DS(data)::DS(data)()
         :easysql_error_reporting(DriverName "_driver",D_IDRV)
         ,connected(false)
{
}

///////////////////////////////////////////////////////////////////////////////
// Finishing that driver instance
//
DS(data)::~DS(data)()
{
}

///////////////////////////////////////////////////////////////////////////////
// Connecting to MySQL server
//
bool
DS(connect) (DS(data) *data, char const *(*values)(void const *, char const *), void const *that)
{ char const *db=values(that,"database");
  if(!db)
   { data->set_error("connect(): database name required");
     return false;
   }

  char const *host=values(that,"host");
  if(host && !strcasecmp(host,"localhost"))
   host=NULL;

  if(!mysql_connect(&(data->sql),host,values(that,"user"),values(that,"password")))
   { data->set_error("connect(host=\"%s\"): %s",host,mysql_error(&(data->sql)));
     return false;
   }

  if(mysql_select_db(&(data->sql),db))
   { data->set_error("connect(db=\"%s\"): %s",db,mysql_error(&(data->sql)));
     return false;
   }

  data->connected=true;
  log(D_IDRV|D_L2,"Connected to MySQL engine (%s,%s)",host,db);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// String, returned when no connection established
//
static char const *noconn_msg="Not connected to engine";

///////////////////////////////////////////////////////////////////////////////
// Checking connection.
//
bool
DS(check_connection) (DS(data) *data)
{ return DS(status)(data)==noconn_msg ? false : true;
}

///////////////////////////////////////////////////////////////////////////////
// Connection and engine status
//
char const *
DS(status) (DS(data) *data)
{ if(!data->connected)
   return noconn_msg;
  return mysql_stat(&(data->sql));
}

///////////////////////////////////////////////////////////////////////////////
// Dropping connection
//
bool
DS(drop) (DS(data) *data)
{ mysql_close(&(data->sql));
  data->connected=false;
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Sending SQL query and returning pointer to results (or NULL in case of
// no results).
//
bool
DS(query) (DS(data) *data, esqlResult &r, char const *q)
{ log(D_IDRV|D_L1,"query(\"%s\")",q);
  r=NULL;

  if(mysql_query(&(data->sql),q))
   { data->set_error("query(\"%s\"): %s",q,mysql_error(&(data->sql)));
     return false;
   }

  r=(esqlResult)mysql_store_result(&(data->sql));
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Freeing query results
//
bool
DS(rfree) (DS(data) *, esqlResult r)
{ mysql_free_result((MYSQL_RES *)r);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Selecting next data record from results
//
bool
DS(row) (DS(data) *, esqlRow &row, esqlResult r)
{ row=(esqlRow)mysql_fetch_row((MYSQL_RES *)r);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Version of driver, api and engine
//
char const *
DS(version) (DS(data) *data)
{ static char *ver=NULL;
  if(!ver) ver=(typeof(ver))mAlloc(100);
  snprintf(ver,100,"%s %s",DriverName,EASYSQL_VERSION);
  if(data->connected)
   { char const *v=(data->sql).server_version;
     if(strlen(v)+strlen(ver)+4<100)
      strcat(strcat(ver," 0 "),v);
   }
  return ver;
}

///////////////////////////////////////////////////////////////////////////////
// Standart functions
//
#include "../DrvStd.h"

///////////////////////////////////////////////////////////////////////////////
R("$Id: mysql.C,v 1.3 1997/09/20 19:05:35 am Exp $");
